import typing as t
from requests import Response


__all__ = [
    'BitpinExceptions',
    'RequestsExceptions',
    'RequestTimeout',
    'InvalidToken',
    'InvalidResponse',
    'StatusCodeError',
    'JSONDecodingError'
]


class BitpinExceptions(Exception):
    def __init__(
            self, func_name: str, message: t.Union[str, t.Type[Exception], Exception], *args, **kwargs
    ):
        """
        Base exception class for Bitpin.

        :param func_name: function name that raise exception
        :type func_name: str

        :param message: message of exception
        :type message: str | Exception

        :param args: args of exception
        :type args: t.Any

        :param kwargs: kwargs of exception
        :type kwargs: t.Any

        :return: None
        :rtype: None
        """

        self.func_name = func_name
        self.message = str(message)
        self.args_ = args
        self.kwargs_ = kwargs
        super().__init__(self.message)

    def __str__(self):
        __str = f'[{self.func_name}] -> {self.message}'

        if self.args_:
            __str += f' | Args: {self.args_}'
        if self.kwargs_:
            __str += f' | Kwargs: {self.kwargs_}'

        return __str


class RequestsExceptions(BitpinExceptions):
    """ Exception class for requests error. """


class RequestTimeout(RequestsExceptions):
    """ Exception class for requests timeout error. """


class InvalidToken(RequestsExceptions):
    """ Exception class for invalid token error. """


class ProcessExceptions(BitpinExceptions):
    """ Exception class for process error. """

    def __init__(self, func_name: str, message: t.Union[str, t.Type[Exception], Exception],
                 response: t.Optional[Response] = None, *args, **kwargs):

        self.func_name = func_name
        self.message = str(message)
        self.args_ = args
        self.kwargs_ = kwargs
        self.response = response

        super().__init__(self.func_name, self.message)

    def __str__(self):
        __str = f'[{self.func_name}] -> {self.message}'

        if self.args_:
            __str += f' | Args: {self.args_}'
        if self.kwargs_:
            __str += f' | Kwargs: {self.kwargs_}'

        return __str


class StatusCodeError(ProcessExceptions):
    """ Exception class for status code error. """


class JSONDecodingError(ProcessExceptions):
    """ Exception class for json decode error. """


class InvalidResponse(ProcessExceptions):
    """ Exception class for invalid response error. """



from enum import Enum


class CurrenciesIDs(Enum):
    IRT = 2
    VTHO = 92
    ALGO = 93
    ATLAS = 148
    BTC = 1
    ETH = 3
    USDT = 4
    BNB = 10
    ADA = 9
    SOL = 51
    XRP = 5
    BCH = 7
    DOT = 8
    USDC = 27
    XLM = 11
    BABYDOGE = 57
    LTC = 6
    XMR = 12
    UNI = 22
    EOS = 13
    LINK = 26
    XEM = 14
    TRX = 15
    XTZ = 16
    THETA = 17
    VET = 18
    ATOM = 19
    NEO = 20
    DASH = 21
    DOGE = 33
    GRT = 37
    FIL = 23
    AAVE = 24
    BUSD = 31
    WAVES = 25
    ZEC = 29
    YFI = 28
    DAI = 34
    GALA = 127
    ETC = 30
    SHIB = 40
    COMP = 32
    SUSHI = 35
    MKR = 36
    BTT = 38
    IOTA = 39
    ONE = 50
    MANA = 54
    CELR = 47
    CHZ = 46
    EGLD = 52
    TWT = 45
    ZIL = 53
    CRV = 144
    MATIC = 44
    CAKE = 43
    ANKR = 55
    FTM = 42
    WIN = 41
    TOMO = 48
    SAFEMOON = 49
    SAND = 62
    MIR = 64
    ONEINCH = 63
    AXS = 67
    AVAX = 68
    ALICE = 66
    ENJ = 65
    REEF = 59
    TLM = 61
    HOT = 60
    KISHU = 56
    FEG = 58
    FORTH = 69
    QNT = 70
    IOTX = 139
    LINA = 71
    DODO = 72
    CTK = 89
    DENT = 88
    CHR = 87
    BEL = 86
    EPS = 85
    C98 = 84
    BAND = 83
    INJ = 82
    XVS = 81
    SXP = 80
    ONT = 79
    BAT = 78
    SNX = 77
    NEAR = 76
    SKL = 75
    LUNA = 90
    MASK = 74
    DYDX = 91
    COTI = 73
    SLP = 96
    FTT = 95
    BNT = 143
    KSM = 94
    AUDIO = 142
    ELON = 128
    MDX = 137
    POLS = 141
    BAKE = 140
    SFM = 149
    BTTC = 150
    BLOK = 147
    CITY = 146
    VRA = 129
    PORTO = 138
    JOE = 145
    PSG = 136
    LAZIO = 135
    CQT = 131
    ATM = 133
    JUV = 134
    ASR = 132
    FLOKI = 130


class MarketsIDs(Enum):
    BTC_IRT = 1
    BTC_USDT = 2
    ETH_USDT = 3
    ETH_IRT = 4
    USDT_IRT = 5
    ADA_USDT = 15
    BNB_IRT = 16
    LINK_USDT = 19
    SOL_USDT = 109
    ADA_IRT = 14
    TRX_USDT = 31
    SOL_IRT = 108
    XRP_IRT = 6
    LTC_USDT = 9
    DOT_USDT = 13
    XRP_USDT = 7
    DOT_IRT = 12
    DOGE_USDT = 63
    DOGE_IRT = 62
    UNI_USDT = 45
    SHIB_IRT = 78
    BNB_USDT = 17
    LUNA_IRT = 178
    USDC_IRT = 20
    XLM_USDT = 23
    AVAX_IRT = 110
    LTC_IRT = 8
    USDC_USDT = 21
    XMR_IRT = 24
    XMR_USDT = 25
    UNI_IRT = 44
    LINK_IRT = 18
    XEM_IRT = 28
    XEM_USDT = 29
    BCH_IRT = 10
    BCH_USDT = 11
    BUSD_IRT = 50
    THETA_USDT = 35
    MATIC_IRT = 86
    MATIC_USDT = 87
    ALGO_IRT = 184
    VET_IRT = 36
    XLM_IRT = 22
    VET_USDT = 37
    AXS_IRT = 113
    ATOM_USDT = 39
    NEO_IRT = 40
    NEO_USDT = 41
    FIL_IRT = 46
    TRX_IRT = 30
    DASH_IRT = 42
    DASH_USDT = 43
    ATOM_IRT = 38
    ETC_IRT = 58
    THETA_IRT = 34
    FIL_USDT = 47
    FTT_IRT = 186
    FTM_IRT = 82
    FTM_USDT = 83
    DAI_IRT = 66
    DAI_USDT = 67
    BUSD_USDT = 51
    EGLD_IRT = 104
    WAVES_IRT = 52
    WAVES_USDT = 53
    NEAR_IRT = 171
    GRT_USDT = 68
    GRT_IRT = 69
    XTZ_IRT = 32
    XTZ_USDT = 33
    ETC_USDT = 59
    EOS_IRT = 26
    EOS_USDT = 27
    GALA_IRT = 221
    GALA_USDT = 222
    CAKE_IRT = 84
    CAKE_USDT = 85
    MANA_IRT = 106
    MANA_USDT = 107
    AAVE_IRT = 48
    AAVE_USDT = 49
    IOTA_USDT = 74
    IOTA_IRT = 75
    SHIB_USDT = 79
    KSM_IRT = 190
    KSM_USDT = 191
    QNT_USDT = 138
    QNT_IRT = 139
    CRV_IRT = 251
    CRV_USDT = 252
    ONE_IRT = 98
    ONE_USDT = 99
    MKR_USDT = 72
    MKR_IRT = 73
    CHZ_IRT = 90
    CHZ_USDT = 91
    EGLD_USDT = 105
    BTT_USDT = 76
    BTT_IRT = 77
    AVAX_USDT = 111
    HOT_USDT = 124
    HOT_IRT = 125
    AXS_USDT = 112
    ENJ_USDT = 130
    ENJ_IRT = 131
    ZEC_IRT = 54
    ZEC_USDT = 55
    COMP_IRT = 60
    COMP_USDT = 61
    SAND_USDT = 116
    SAND_IRT = 117
    BAT_USDT = 166
    BAT_IRT = 167
    IOTX_IRT = 245
    IOTX_USDT = 246
    SUSHI_USDT = 70
    SUSHI_IRT = 71
    ZIL_IRT = 102
    ZIL_USDT = 103
    YFI_IRT = 56
    YFI_USDT = 57
    SNX_USDT = 168
    SNX_IRT = 169
    NEAR_USDT = 170
    LUNA_USDT = 179
    ALGO_USDT = 185
    BNT_IRT = 253
    BNT_USDT = 254
    ANKR_IRT = 100
    ANKR_USDT = 101
    FTT_USDT = 187
    ONT_USDT = 164
    ONT_IRT = 165
    DYDX_IRT = 180
    DYDX_USDT = 181
    SKL_USDT = 172
    SKL_IRT = 173
    ONEINCH_USDT = 132
    ONEINCH_IRT = 133
    ELON_IRT = 223
    ELON_USDT = 224
    CELR_IRT = 92
    CELR_USDT = 93
    WIN_IRT = 80
    WIN_USDT = 81
    DENT_USDT = 146
    DENT_IRT = 147
    AUDIO_IRT = 255
    AUDIO_USDT = 256
    ALICE_USDT = 114
    ALICE_IRT = 115
    INJ_USDT = 158
    INJ_IRT = 159
    CHR_USDT = 148
    CHR_IRT = 149
    SXP_USDT = 162
    SXP_IRT = 163
    REEF_USDT = 128
    REEF_IRT = 129
    MDX_IRT = 241
    MDX_USDT = 242
    COTI_USDT = 176
    COTI_IRT = 177
    VTHO_IRT = 182
    VTHO_USDT = 183
    POLS_IRT = 249
    POLS_USDT = 250
    BAKE_IRT = 247
    BAKE_USDT = 248
    FLOKI_IRT = 227
    FLOKI_USDT = 228
    BLOK_IRT = 261
    BLOK_USDT = 262
    BTTC_USDT = 267
    BTTC_IRT = 268
    SAFEMOON_USDT = 96
    SAFEMOON_IRT = 97
    C98_USDT = 154
    C98_IRT = 155
    MASK_USDT = 174
    MASK_IRT = 175
    TWT_IRT = 88
    TWT_USDT = 89
    TLM_IRT = 126
    TLM_USDT = 127
    XVS_USDT = 160
    XVS_IRT = 161
    SFM_USDT = 266
    TOMO_IRT = 94
    TOMO_USDT = 95
    MIR_USDT = 134
    MIR_IRT = 135
    BAND_USDT = 156
    BAND_IRT = 157
    VRA_IRT = 225
    VRA_USDT = 226
    LINA_USDT = 140
    LINA_IRT = 141
    EPS_USDT = 152
    EPS_IRT = 153
    SLP_IRT = 188
    SLP_USDT = 189
    CTK_USDT = 144
    CTK_IRT = 145
    FORTH_USDT = 136
    FORTH_IRT = 137
    DODO_USDT = 142
    DODO_IRT = 143
    BEL_USDT = 150
    BEL_IRT = 151
    PORTO_IRT = 243
    PORTO_USDT = 244
    PSG_IRT = 239
    PSG_USDT = 240
    LAZIO_USDT = 237
    LAZIO_IRT = 238
    CQT_IRT = 229
    CQT_USDT = 230
    ATM_IRT = 234
    ATM_USDT = 233
    JUV_IRT = 235
    JUV_USDT = 236
    ASR_IRT = 231
    ASR_USDT = 232
    BABYDOGE_USDT = 120
    BABYDOGE_IRT = 121
    KISHU_USDT = 118
    KISHU_IRT = 119
    FEG_USDT = 122
    FEG_IRT = 123
    SFM_IRT = 265
    CRO_IRT = 312



import typing as t
import requests

from inspect import currentframe as cf


from .exceptions import *
from .helpers import CurrenciesIDs, MarketsIDs

from logging import Logger

from nice_tools.thread_tools import run_in_thread_decorator
from time import sleep

from redis import Redis

__all__ = [
    'Bitpin',
]


class Bitpin:
    def __init__(self, db_params: t.Optional[t.Dict] = {}, requests_params: t.Optional[t.Dict] = None,
        logger: t.Optional[t.Union[Logger, t.Type[Logger]]] = Logger('Bitpin')):
        """
        Initialize the Bitpin SDK .

        :param api_key: API Key (optional)
        :type api_key: str

        :param api_secret: API Secret (optional)
        :type api_secret: str

        :param requests_params: Requests parameters (optional)
        :type requests_params: dict
        """

        self.__headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
        }

        self.__base_url = 'https://api.bitpin.ir'

        self._logger = logger

        self._redis = Redis(**db_params, decode_responses=True)

        # set timeout from requests_params
        if requests_params is not None:
            self.__timeout = requests_params.get('timeout') or 10
            self.__verify = requests_params.get('verify') or None
        else:
            self.__timeout = 10
            self.__verify = True
    
    def get_token(self) -> t.Union[str, None]:
        return self._redis.get('access_token')

    @staticmethod
    def __get_func_args(all_locals: t.Dict, remove_keys=None) -> t.Dict:
        """
        Get function arguments.

        :param all_locals: All locals
        :type all_locals: dict

        :return: Function arguments
        :rtype: dict
        """

        if remove_keys is None:
            remove_keys = ['self']

        if 'self' not in remove_keys:
            remove_keys.append('self')

        for i in remove_keys:
            try:
                all_locals.pop(i)
            except KeyError:
                pass

        return all_locals

    def __make_url(self, url: str) -> str:
        """
        Make url.

        :param url: Url
        :type url: str

        :return: Url
        :rtype: str
        """

        if url.startswith('/'):
            url = url[1:]

        if not url.endswith('/'):
            url += '/'

        return f'{self.__base_url}/{url}'

    def __make_headers(self, func_name: str, auth: t.Optional[bool] = True) -> t.Dict:
        """
        Make headers.

        :param func_name: Function name
        :type func_name: str

        :param auth: Authorization (optional)
        :type auth: bool

        :return: Headers
        :rtype: dict
        """

        f_args = self.__get_func_args(locals())

        headers = self.__headers.copy()

        if auth is True:
            access_token = self.get_token()
            if access_token is None:
                raise InvalidToken(
                    func_name,
                    'access_token is None',
                    f_args=f_args
                )
            headers.update({
                'Authorization': f'Bearer {access_token}'
            })

        return headers

    def _raise_for_exception(
            self,
            func_name: str,
            response: requests.Response,
            additional: t.Dict = None
    ) -> None:
        """
        Check response for exceptions.

        :param response: Response
        :type response: requests.Response

        :param func_name: Function name
        :type func_name: str

        :raises: NobitexAPIException

        :return: None
        :rtype: None
        """

        additional.update(self.__get_func_args(locals()))

        if 200 <= response.status_code < 300:
            try:
                if response.request.method.lower() == 'delete':
                    return
                r_json: t.Dict = response.json()
            except Exception as e:
                raise JSONDecodingError(func_name, e, response=response, additional=additional)

        else:
            raise StatusCodeError(
                func_name,
                f'invalid status code {response.text}',
                url=response.url,
                status_code=response.status_code,
                response=response,
                additional=additional
            )

    def _process_response(
            self,
            func_name: str,
            response: requests.Response,
            additional: t.Dict = None,
    ) -> t.Dict:
        """
        Process the response from the Nobitex API.

        :param response: Response
        :type response: requests.Response

        :param func_name: Function name
        :type func_name: str

        :param additional: Arguments (optional)
        :type additional: dict

        :raises: NobitexAPIException

        :return: Response
        :rtype: dict
        """

        try:
            self._raise_for_exception(func_name, response, additional)
        except Exception as e:
            self._logger.error(f'{response.request.url} failed {e}')
            raise e
        if response.request.method.lower() == 'delete':
            return {'success': True}
        self._logger.info(f'{response.request.url} success {response.json()}')
        return response.json()

    def __request(
            self, method: str, url: str, headers: t.Dict = None,
            params: t.Dict = None, data: t.Dict = None, json_data: t.Dict = None, additional_kwargs: t.Dict = None
    ) -> requests.Response:
        """
        Send request to the Bitpin API.

        :param method: Method
        :type method: str

        :param url: URL
        :type url: str

        :param headers: Headers (optional)
        :type headers: dict

        :param params: Parameters (optional)
        :type params: dict

        :param data: Data (optional)
        :type data: dict

        :param json_data: JSON data (optional)
        :type json_data: dict

        :raises: BitpinAPIException

        :return: Response
        :rtype: requests.Response
        """

        f_name = cf().f_code.co_name

        try:
            response = requests.request(
                method,
                self.__make_url(url),
                headers=headers,
                params=params,
                json=json_data,
                data=data,
                timeout=self.__timeout
            )

            return response
        except Exception as e:
            raise RequestsExceptions(f_name, e, additional_data=additional_kwargs)

    def _request(
            self, func_name: str, method: str, url: str, auth: bool = False,
            params: t.Dict = None, data: t.Dict = None, json_data: t.Dict = None,
    ) -> requests.Response:
        """
        Make a request to the Bitpin API.

        :param method: HTTP method
        :type method: str

        :param url: URL
        :type url: str

        :param auth: Whether to use authentication (optional)
        :type auth: bool

        :param params: Query parameters (optional)
        :type params: dict

        :param data: Request body (optional)
        :type data: dict

        :param json_data: Request body (optional)
        :type json_data: dict

        :param func_name: Function name
        :type func_name: str

        :return: Response
        :rtype: requests.Response
        """

        f_args = self.__get_func_args(locals())

        headers = self.__make_headers(func_name, auth)

        return self.__request(method.upper(), url, headers, params, data, json_data)

    def login(self, api_key: str, secret_key: str) -> t.Dict:
        """
        Get token.

        :return: Refresh token and access token
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/usr/api/login/'

        json_data = {
            'api_key': api_key,
            'secret_key': secret_key,
        }

        response = self._request(
            f_name,
            'POST',
            url,
            auth=False,
            json_data=json_data
        )

        return self._process_response(f_name, response, f_args)

    def refresh_token(self) -> t.Dict:
        """
        Refresh token.
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/usr/refresh_token/'

        json_data = {
            'refresh': self.__refresh_token,
        }

        response = self._request(
            f_name,
            'POST',
            url,
            auth=False,
            json_data=json_data,
        )

        return self._process_response(f_name, response, f_args)

    def user_info(self) -> t.Dict:
        """
        Get user info.

        :return: User info
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/usr/info/'

        response = self._request(
            f_name,
            'GET',
            url,
            auth=True,
        )

        return self._process_response(f_name, response, f_args)

    def currencies_list(self, page: t.Optional[int] = 1) -> t.Dict:
        """
        Get currencies list.

        :param page: Page number (optional)
        :type page: int

        :return: Currencies list
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/mkt/currencies/'

        params = {
            'page': page,
        }

        response = self._request(
            f_name,
            'GET',
            url,
            auth=False,
            params=params,
        )

        return self._process_response(f_name, response, f_args)

    def markets_list(self, page: t.Optional[int] = 1) -> t.Dict:
        """
        Get markets list.

        :param page: Page number (optional)
        :type page: int

        :return: Markets list
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/mkt/markets/'

        params = {
            'page': page,
        }

        response = self._request(
            f_name,
            'GET',
            url,
            auth=False,
            params=params,
        )

        return self._process_response(f_name, response, f_args)

    def wallets(self) -> t.Dict:
        """
        Get wallets.

        :return: Wallets
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/wlt/wallets/'

        response = self._request(
            f_name,
            'GET',
            url,
            auth=True,
        )

        return self._process_response(f_name, response, f_args)

    def orderbook(self, market_id: t.Union[int, MarketsIDs], side: t.Optional[str] = None) -> t.Dict:
        """
        Get orderbook.

        :param market_id: Market ID
        :type market_id: int

        :param side: Side (optional)
        :type side: str

        :return: Orderbook
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        if type(market_id) is MarketsIDs:
            market_id = market_id.value

        url = f'/v2/mth/actives/{market_id}/'

        side = side.lower() if side else None

        if side and side not in ['buy', 'sell']:
            raise BitpinExceptions(
                f_name,
                'Invalid side',
                f_args=f_args
            )
        else:
            params = {
                'type': side,
            }

        response = self._request(
            f_name,
            'GET',
            url,
            auth=False,
            params=params,
        )

        return self._process_response(f_name, response, f_args)

    def latest_trades(self, market_id: t.Union[int, CurrenciesIDs]) -> t.Dict:
        """
        Get latest trades.

        :param market_id: Market ID
        :type market_id: int

        :return: Latest trades
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        if type(market_id) is CurrenciesIDs:
            market_id = market_id.value

        url = f'/v1/mth/matches/{market_id}/'

        response = self._request(
            f_name,
            'GET',
            url,
            auth=False,
        )

        return self._process_response(f_name, response, f_args)

    def create_order(self, market: int, order_type: str, side: str, amount: float,
                     price: float = None, order_id: str = None) -> t.Dict:
        """
        Create order.

        :param market: Market ID (required)
        :type market: int

        :param order_type: Order type (required)
        :type order_type: str

        :param side: Side (required)
        :type side: str

        :param amount: Amount (required)
        :type amount: float

        :param price: Price (optional)
        :type price: float

        :param order_id: Order ID (optional)
        :type order_id: str

        :return: Order
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        if order_type.lower() == 'limit' and price is None:
            raise

        url = '/v1/odr/orders/'

        json_data = {
            'market': market,
            'mode': order_type,
            'type': side,
            'amount1': amount,
            'price': price,
            'price_limit': price,
        }

        if order_id:
            json_data['identifier'] = order_id

        response = self._request(
            f_name,
            'POST',
            url,
            auth=True,
            json_data=json_data
        )

        return self._process_response(f_name, response, f_args)

    def cancel_order(self, order_id: int) -> t.Dict:
        """
        Cancel order.

        :param order_id: Order ID (required)
        :type order_id: int

        :return: Order
        :rtype: dict
        """
        
        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = f'v1/odr/orders/{order_id}/'

        response = self._request(
            f_name,
            'DELETE',
            url,
            auth=True,
        )

        return self._process_response(f_name, response, f_args)
    
    def order_history(self):
        """
        Get order history.

        :return: Recent trades
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/odr/orders/'

        response = self._request(
            f_name,
            'GET',
            url,
            auth=True,
        )

        return self._process_response(f_name, response, f_args)

    def recent_trades(self, market_id: t.Optional[int] = None, page: int = None) -> t.Dict:
        """
        Get recent trades.

        :param market_id: Market ID (optional)
        :type market_id: int

        :param page: Page number (optional)
        :type page: int

        :return: Recent trades
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/odr/matches/'

        params = None

        if page is not None:
            params = {'page': page}

        response = self._request(
            f_name,
            'GET',
            url,
            auth=True,
            params=params
        )

        processed_response = self._process_response(f_name, response, f_args)

        if market_id is not None:
            filtered_results = [r for r in processed_response.get('results') if r['market']['id'] == market_id]

            processed_response['results'] = filtered_results
            
        return processed_response

    def balances(self, currency_id: t.Optional[int] = None) -> t.Dict:
        """
        Get balances.

        :param currency_id: Currency ID (optional)
        :type currency_id: int

        :return: Recent trades
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/wlt/wallets/'

        response = self._request(
            f_name,
            'GET',
            url,
            auth=True,
        )

        processed_response = self._process_response(f_name, response, f_args)

        if currency_id is not None:
            filtered_results = [r for r in processed_response.get('results') if r['currency']['id'] == currency_id]

            processed_response['results'] = filtered_results
            
        return processed_response

    def open_orders(self, market_id: t.Optional[int] = None, page: int = None) -> t.Dict:
        """
        Get open orders.

        :param market_id: Market ID (optional)
        :type market_id: int

        :param page: Page number (optional)
        :type page: int

        :return: Open orders
        :rtype: dict
        """

        f_name = cf().f_code.co_name
        f_args = self.__get_func_args(locals())

        url = '/v1/odr/orders/?state=active,initial'

        params = None

        if page is not None:
            params = {'page': page}

        response = self._request(
            f_name,
            'GET',
            url,
            auth=True,
            params=params,
        )

        processed_response = self._process_response(f_name, response, f_args)

        if market_id is not None:
            filtered_results = [r for r in processed_response.get('results') if r['market']['id'] == market_id]

            processed_response['results'] = filtered_results
            
        return processed_response
